##
# Copyright (c) 2011-2012,2014-2017,2019-2020 SIOS Technology Corp.
##
########################################################################
#
# Title:       IP interface common functions for IP Recovery Kit
#
######################## LifeKeeper Linux CHANGE LOG ###################
#
# CR            date    user    description
# ==            ====    ====    ==========
# bug1760       13Jun11  mat    IPv6 Capable IP RK
# bug1804       17Jun11  mat    fix quickCheck timeout
# bug1822       28Jun11  sra    Add App_delete()
# bug----       28Jun11  sra    Fix path to srcaddr script
# bug----       27Jul11  sra    Add more dbg) calls for better tracing
# bug1954       01Aug11  sra    Pass the netmask instead of subnet addr to srcaddr
# bug1894       02Aug11  mat    Add do bcast ping if ping list failed
# bug1984       04Aug11  mat    Add check NIC link state when restore
# bug1984       04Oct11  mat    Add flush arp table and retry link check
# bug2639       30Mar12  yuh    Convert logging to LogMsg
# bug2795       30Apr12  stt    Changed a log message to only logfile to avoid dups.
# bug363        04May12  stt    Added additional logging around ping failures inc. bcast pings.
# bug3567       03Oct14  mas    Fix return broadcast address
# bug3744       14Jan15  numa   add sleep between ifdown and ifup when NIC link is down
# bug7073       27Mar15  matu   Change subroutin to apropriate for wrong output message about format string
# bug7122       10Jul15  ryo    Change initial value for same behavior as default without config
# bug7263       18Oct16  jef    Correctly handle bonded interfaces when link is down
# bug7315       13Feb17  kshi   Support Network teaming device
# bug7322       01Mar17  ryo    Clarify specifications of parameters IP_PINGTIME and IP_PINGTRIES
# bug7324       14Mar17  ysa    delete restore-mode flag in resource remove
# bug7325       16Mar17  ryo    Changed to use the signal handler defined by RKActionHandler.pm
# bug3602       24Mar17  ryo    Show error message when virtual IP is not configured in check()
# bug7357       27JUL17  ryo    Real IP resource support
# bug7636	17May19  kei    use /32 mask on AWS
# bug7838       07NOV19  osa    Local recovery for IP resource fails on RHEL8
# PL-3760	20NOV20  kei	use both ifup and ifconfig up when resotre
# PL-7858	17Dec20  jef	Fix typo in ifconfig up stderr redirection
#


package IPint;
BEGIN { require '/etc/default/LifeKeeper.pl'; @ISA = qw(RKBase);}

use Data::Dumper;
use File::Basename;

use RKBase;
use LKIP;
use LK;
use IPutil;
use IPintv4;
use IPintv6;
use Fcntl;
use strict;

our $Logs = {'STDERR' => 1, 'LOGFILE' => 1};

$0 =~ s/^.*[\/]//g;

sub new {
	my $type = shift;
	my $self = shift; # This is really a ref to the superclass

	bless $$self, $type;

	init($self); # Pass the ref

	return $$self;
}

#
# NB: This is usually created from an RK* object. RKActionHandler will reset the bless & ISA, so we can't
# actually subclass this any further. Instead of subclassing IPintv4 and IPintv6, we'll just create
# small objects for them, and call through this object to the methods on those objects.
#
sub init {
	my $self = shift; # A ref to $self

	#
	# Grab the stuff that has been passed to us
	#
	$$self->{'id'} = $$self->Val('ID');

	if ($$self->{'id'}) {
		# This test is for the convenience of unit test scripts
		$$self->{'ip'} = $$self->{'id'}; # IP address is ID
	}
	$$self->{'ip'} =~ s/IP-//; # Cut off any prefix
	$$self->{'ip'} =~ s/IP6-//; # Cut off any prefix
	$$self->{'tag'} = $$self->Val('Tag');
	$$self->{'sys'} = $$self->Val('SysName');
	$$self->{'state'} = $$self->Val('State');
	if( $$self->Val('Info') ) {
		# This test is for the convenience of unit test scripts
		$$self->{'info'} = $$self->Val('Info');
	}

	# Set some defaults
	$$self->{'deadline'} = $ENV{IP_PINGTIME} || 1;
	$$self->{'pingtries'} = $ENV{IP_PINGTRIES} || 3;
	$$self->{'quick_check_timeout'} = $ENV{IP_QUICKCHECK_TIMEOUT} || 12;
	$$self->{'no_ip_unique'} = $ENV{NOIPUNIQUE} || 0;
	$$self->{'no_bcast_ping'} = $ENV{NOBCASTPING} || 0;
	$$self->{'no_link_check'} = $ENV{IP_NOLINKCHECK} || 0;
	$$self->{'max_linkchk'} = $ENV{IP_MAX_LINKCHK} || 5;

	# Decide which flavor of IP this is
	$$self->{'ipversion'} = LKIP::ip_get_version($$self->{'ip'});

	# Create a sub-object of the right flavor
	if ( $$self->{'ipversion'} == '4' ) {
		$$self->{'ipint'} = new IPintv4();
	}
	elsif ( $$self->{'ipversion'} == '6' ) {
		$$self->{'ipint'} = new IPintv6();
	}
	else {
		LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_DEBUG, 'ip', $0, $$self->{'tag'}, 123005, "Dumper($$self)");
		LKBase::LogMsgAndDie($Logs, LK::LK_FATAL, 'ip', $0, $$self->{'tag'}, 123006, 'Unknown version %s of IP address', $$self->{'ipversion'});

	}

	# Do parsing of the Info
	# [0]host, [1]device, [2]addr, [3]netmask, [4] undef, [5] srcaddr, [6] prefix
	@_ = IPutil::parse_info($$self->{'info'});
	$$self->{'device'} = $_[1];
	$$self->{'ipaddr'} = $_[2];
	$$self->{'netmask'} = $_[3];
	$$self->{'src_addr'} = $_[5];
	$$self->{'prefix'} = $_[6];
	$$self->{'realip'} = main::getkey($$self->{'info'}, 'realip', ' ');

	# Create the subnet address and ping list file name
	$$self->{'subnet'} = IPutil::get_subnet($$self->{'ipaddr'}, $$self->{'netmask'});
	$$self->{'plist_path'} = "$ENV{'LKROOT'}/subsys/comm/resources/ip/pinglist." . $$self->{'subnet'};
	$$self->{'resmode_path'} = "$ENV{'LKROOT'}/subsys/comm/resources/ip/.resMode-" . $$self->{'id'};

	# Set restore mode flag
	my $ID = $$self->{'id'};
	if ( -f $$self->{'resmode_path'} ) {
		$$self->{'restore_mode'} = 1;
	} else {
		$$self->{'restore_mode'} = 0;
	}

	# Make a copy of some things to sub-object for later use
	$$self->{'ipint'}->{'deadline'} = $$self->{'deadline'};
	$$self->{'ipint'}->{'pingtries'} = $$self->{'pingtries'};
	$$self->{'ipint'}->{'device'} = $$self->{'device'};
	$$self->{'ipint'}->{'ip'} = $$self->{'ip'};
	$$self->{'ipint'}->{'subnet'} = $$self->{'subnet'};
	$$self->{'ipint'}->{'netmask'} = $$self->{'netmask'};
	$$self->{'ipint'}->{'ipversion'} = $$self->{'ipversion'};
	$$self->{'ipint'}->{'plist_path'} = $$self->{'plist_path'};

	# Set broadcast address/all nodes multicast address
	$$self->{'bcast_addr'} = $$self->{'ipint'}->get_bcast_addr;
	$$self->{'ipint'}->{'bcast_addr'} = $$self->{'bcast_addr'};

	# Misc stuff, which needs further refinement

	# Set NetworkManager status
	$$self->{'rh_network_manager'} = 0;
	if ($ENV{'LK_DISTRIBUTION'} ne "suse" && -x "/usr/bin/systemctl") {
		if (system("systemctl is-active NetworkManager > /dev/null 2>&1") == 0) {
			$$self->{'rh_network_manager'} = 1;
		}
	}
}


sub DESTROY {
	my $self = shift;
#print STDERR "in IPint::DESTROY\n";
#print STDERR Dumper($self);
}

#
# A set of methods for determining if an IP is in use
#

sub execute_list_ping {
	my $self = shift;
	my @list;
	my $ret;

	LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_DEBUG, 'ip', $0, $self->{'tag'}, 123007, "in IPint::execute_list_ping()");
	sysopen(my $fh, "$self->{'plist_path'}", O_RDONLY) || do {
		$self->vLogMsg(LK::LK_ERROR, 'ip', $0, 123008, 'No pinglist found for %s.', $self->{'ipaddr'});
		return 1;
	};
	@list = <$fh>;
	close($fh);

	foreach my $tgt (@list) {
		$ret = $self->{'ipint'}->do_ping($tgt, "-I $self->{'ip'}");
		return 1 if $ret;
	}

	$self->vLogMsg(LK::LK_ERROR, 'ip', $0, 123009, 'List ping test failed for virtual IP %s', $self->{'ipaddr'});

	if (! ($self->{'no_bcast_ping'})) {
		$self->vLogMsg(LK::LK_INFO, 'ip', $0, 123010, 'Falling into the broadcast ping test');
		$ret = $self->{'ipint'}->execute_bcast_ping;
		if (!$ret) {
			LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_ERROR, 'ip', $0, $self->{'tag'}, 123097, 'exec_list_ping(): broadcast ping failed.');
		}
	}
	return $ret;
}

#
# check NIC link up
#
# Link up: return 0, Link down: return 1
sub is_link_down {
	my $self = shift;
	my $dev = $self->{'device'};

	LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_DEBUG, 'ip', $0, $self->{'tag'}, 123011, "in IPint::is_link_down()");
	if ($self->{'no_link_check'}) {
		# no check
		LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_DEBUG, 'ip', $0, $self->{'tag'}, 123012, "no check link down: IP_NOLINKCHECK = $self->{'no_link_check'}");
		return 0;
	}

	my $lnk = `LC_ALL=C ip -o link show $dev`;
	if ( $? != 0 ||  $lnk =~ /NO-CARRIER/ || $lnk !~ /\bUP\b/ ) {
		$self->vLogMsg(LK::LK_ERROR, 'ip', $0, 123013, 'Link check failed for virtual IP %s on interface %s.', $self->{'ip'}, $dev);
		return 1;
	}

	# If NetworkManager is active on non-suse env, IFF_UP is always set.
	# Check the device if it is really active.
	if ($self->{'rh_network_manager'} == 1) {
		LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_DEBUG, 'ip', $0, $self->{'tag'}, 123088, "nmcli connection show --active | grep -qE \'\\s\+$dev\\s\*\'");
		if (system("nmcli connection show --active | grep -qE \'\\s\+$dev\\s\*\'") != 0) {
			$self->vLogMsg(LK::LK_ERROR, 'ip', $0, 123089, 'Link check failed for virtual IP %s on interface %s.', $self->{'ip'}, $dev);
			return 1;
		}
	}

	# check slave NIC of Bonding
	my $down_slave = 0;
	my @slaves = (`LC_ALL=C ip -o link show \| grep " master $dev " 2>/dev/null`);
	return 0 if (@slaves == 0); # return if $dev is not bonding/teaming device
	map {
		my $nic=(split/:/)[1] if ( /NO-CARRIER/ || $_ !~ /\bUP\b/ );
		if ($nic) {
			LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_DEBUG, 'ip', $0, $self->{'tag'}, 123014, "Slave NIC $nic is down");
			$down_slave++;
		}
	} @slaves;
	if ( @slaves == $down_slave ) {
		$self->vLogMsg(LK::LK_ERROR, 'ip', $0, 123015, 'Link check failed for virtual IP %s on interface %s.', $self->{'ip'}, $dev);
		return 1;
	}
	return 0;
}

# Checks the validity of the IP managed by this instance.  This routine
# may try various tests to get that done.
#
# Returns ($result, $tried_bcast) where $result is 0 for all-good and non-zero
# for errors, $tried_bcast is a bool that indicates whether or not
# a broadcast ping was attempted in the process of checking.  $result is usually
# all that's needed but both values are returned.
sub check {
	my $self = shift;
	my $dev = $self->{'device'};
	my $ret = 1;
	my $tried_bcast = 0;

	LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_DEBUG, 'ip', $0, $self->{'tag'}, 123016, "in IPint::check()");
	if (! $self->is_ip_configured) {
		$self->vLogMsg(LK::LK_ERROR, 'ip', $0, 123090, 'virtual IP %s on interface %s is not configured', $self->{'ip'}, $dev);
		# False: return 1.
		return (1, 0);
	}

	if ($self->is_link_down) {
		$self->vLogMsg(LK::LK_ERROR, 'ip', $0, 123091, 'the link for interface %s is down', $dev);
		# Link down
		return (1, 0);
	}

	if ( -f "$self->{'plist_path'}" ) {
		# ping to target ip from vip.
		$ret = $self->execute_list_ping;
		if (!$ret) {
			LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_ERROR, 'ip', $0, $self->{'tag'}, 123093, 'the ping list check failed');
		}
	} elsif ($self->{'no_bcast_ping'}) {
		# don't broadcast/multicast ping
		return (0, 0);
	} else {
		# ping to broadcast or all nodes multicast (ff02::1)
		LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_DEBUG, 'ip', $0, $self->{'tag'}, 123017, "IPint::check(): doing broadcast ping");
		$ret = $self->{'ipint'}->execute_bcast_ping;
		if (!$ret) {
			LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_ERROR, 'ip', $0, $self->{'tag'}, 123095, 'broadcast ping failed');
		}
		$tried_bcast = 1;
	}

	# OK: return 0, False: return 1.
	return (! $ret, $tried_bcast);
}

sub ip_exists {
	my $self = shift;
	my $ret;

	LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_DEBUG, 'ip', $0, $self->{'tag'}, 123018, "ip_exists(".$self->{'ip'}.")");

	# if dont get reply, there is a problem w/ the IP
	$ret = $self->{'ipint'}->do_ping("$self->{'ip'}");
	return $ret;

}

#
# See if an IP is currently configured on a device
#
sub is_ip_configured {
	my $self = shift;
	my $addr = LKIP::ip_expand_address($self->{'ip'},$self->{'ipversion'});
	my $ret;

	LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_DEBUG, 'ip', $0, $self->{'tag'}, 123019, "in IPint::is_ip_configured()");
	my @iplist = IPutil::get_ip_dev($self->{'device'}, $self->{'ipversion'});

	$ret = grep /^$addr$/, @iplist;
	LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_DEBUG, 'ip', $0, $self->{'tag'}, 123020, "is_ip_configured($addr) $ret");
	return ($ret == 0 ? 0 : 1);
}

sub srcaddr {
	my $self = shift;
	my $src_set = shift || 0;

	my $ipbin = "$ENV{'LKROOT'}/lkadm/subsys/comm/ip/bin";
	my $addr = $self->{'ip'};
	my $mask = $self->{'netmask'};
	my $info = $self->{'info'};

	if ($src_set) {
		system("$ipbin/srcaddr -a $addr -m $mask -i \"$info\" -y");
	} else {
		system("$ipbin/srcaddr -a $addr -m $mask -n");
	}
}

#
# Load network interface info into a hash for reference
#
sub load_ifcfg_info {
	my $self = shift;
	my $ifcfg_path;
	my $ifcfg_loopback;
	my $path;
	my $cfg = "ifcfg-";
	my $ifcfg_base_name;
	my $ifcfg_name;
	my $line;
	my $key;
	my $value;
	my %ifcfg_hash = ();

	if (-d "/etc/sysconfig/network-scripts") {
		$ifcfg_path = "/etc/sysconfig/network-scripts/ifcfg*";
		$ifcfg_loopback = "/etc/sysconfig/network-scripts/ifcfg-lo";
	} else {
		$ifcfg_path = "/etc/sysconfig/network/ifcfg*";
		$ifcfg_loopback = "/etc/sysconfig/network/ifcfg-lo";
	}

	foreach $path (glob($ifcfg_path)) {
		chomp($path);
		# Skip loopback config file
		next if ($path =~ /$ifcfg_loopback/);
		$ifcfg_base_name = basename($path);
		$ifcfg_base_name =~ /^$cfg(\S+)$/;
		$ifcfg_name = $1;

		sysopen(my $fh, $path, O_RDONLY) || do {
			$self->vLogMsg(LK::LK_ERROR, 'ip', '', 123299, 'Unable to open %s. Reason %s', $path, $!);
			return undef;
		};
		my @interface_info = <$fh>;
		close ($fh);

		foreach $line (@interface_info) {
			next if ($line =~ /^\s*#/);	# skip comment lines
			next if ($line =~ /^\s*$/);	# skip empty lines
			chomp($line);
			($key, $value)=split /=/, $line;
			uc($key);
			$ifcfg_hash{$ifcfg_name}{$key} = $value;
		}
	}

	return \%ifcfg_hash;
}

#
# Recycle the bonded/teamd interface (take it down and bring back up).  This must be
# done in the correct order or the interface will remain in a failed state.
# The proper order is as follow:
#	1. Slave interfaces down
#	2. Master interface down
#	3. Slave interfaces up
#	4. Master interfaces up
#
sub recycle_bonded {
	my $self = shift;
	my $dev = $self->{'device'};
	my $ref_ifcfg_info;
	my $ifcfg;
	my $master_dev;
	my $master_uuid;
	my $master_ifcfg_file;
	my @slaves;
	my $wait = $ENV{'IP_WAIT_LINKDOWN'} || 5;

	$ref_ifcfg_info = load_ifcfg_info();
	return 1 if (!defined $ref_ifcfg_info);

	# Find the master device, uuid and ifcfg file
	foreach $ifcfg (keys %$ref_ifcfg_info) {
		if ($ref_ifcfg_info->{$ifcfg}{DEVICE} =~ /^$dev$/ &&
		    (uc($ref_ifcfg_info->{$ifcfg}{BONDING_MASTER}) =~ /^YES$/
		      || uc($ref_ifcfg_info->{$ifcfg}{DEVICETYPE}) =~ /^Team$/)) {
			$master_dev = $ref_ifcfg_info->{$ifcfg}{DEVICE};
			$master_uuid = $ref_ifcfg_info->{$ifcfg}{UUID};
			$master_ifcfg_file = $ifcfg;
			last;
		}
	}

	# Now find all the ifcfg files for the slave devices
	foreach $ifcfg (keys %$ref_ifcfg_info) {
		if ((uc($ref_ifcfg_info->{$ifcfg}{SLAVE}) =~ /^YES$/
			&& ( $ref_ifcfg_info->{$ifcfg}{MASTER} =~ /^$master_uuid$/
			|| $ref_ifcfg_info->{$ifcfg}{MASTER} =~ /^$master_dev$/ ) )
		      || ((uc($ref_ifcfg_info->{$ifcfg}{TEAM_MASTER}) =~ /^$master_dev$/ || uc($ref_ifcfg_info->{$ifcfg}{TEAM_MASTER}) =~ /^$master_uuid$/) && uc($ref_ifcfg_info->{$ifcfg}{DEVICETYPE}) =~ /^TeamPort$/)) {
			push @slaves, $ifcfg;
		}
	
	}

	# Down the slaves first
	foreach (@slaves) {
		LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_DEBUG, 'ip', $0, $self->{'tag'}, 123400, "Downing slave interface $_");
		system("ifdown $_ >/dev/null 2>&1 ");
	}

	# Then down the master
	LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_DEBUG, 'ip', $0, $self->{'tag'}, 123401, "Downing master interface $dev");
	system("ifdown $dev >/dev/null 2>&1 ");

	sleep $wait; # VLAN environment require this sleep. bug3744.

	# Bring up the slaves
	foreach (@slaves) {
		LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_DEBUG, 'ip', $0, $self->{'tag'}, 123402, "Upping slave interface $_");
		system("ifup $_ >/dev/null 2>&1 ");
	}

	# Finally bring up the master
	LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_DEBUG, 'ip', $0, $self->{'tag'}, 123403, "Upping master interface $dev");
	system("ifup $dev >/dev/null 2>&1 ");

}


sub on_aws {
	 return !system("curl -m 5 -qf http://169.254.169.254/latest/meta-data/instance-id 2>/dev/null | egrep -q i-");
}

#
#
# These are the callbacks from the superclass (RK*)
#
#

sub App_restore {
	my $self = shift;
	my $dev = $self->{'device'};
	my $ret;

	$self->{'prefix'} = 32 if(on_aws());

	# check real ip resource
	if ($self->{'realip'}) {
		LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_DEBUG, 'ip', $0, $self->{'tag'}, 123098, "real ip resource");
		return 0;
	}

	# check restore mode
	if ($self->{'restore_mode'}) {
		LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_DEBUG, 'ip', $0, $self->{'tag'}, 123021, "restore_mode is on");
		return 0;
	}

	# NIC link check
	if ($self->is_link_down) {
		my $wait = $ENV{'IP_WAIT_LINKDOWN'} || 5;
		LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_DEBUG, 'ip', $0, $self->{'tag'}, 123022, "Trying NIC link down then up for $dev");
		# NIC link up
		# Check if $dev is a bonded/teamd device and we are running on a RedHat based OS.
		# If so then bonded/teamd interfaces require having the slaves taken down before
		# the master is taken down and then having them brought up before the master
		# is brought up.  On SLES the master has the slave info so the bonded/teamd 
		# interface can be take down and back up without anything being done to the
		# slaves.
		my @slaves = (`LC_ALL=C ip -o link show \| grep " master $dev " 2>/dev/null`);
		if (@slaves > 0 && ($ENV{'LK_DISTRIBUTION'} ne "suse") ) {
			# bonded/teamd device on RedHat or RedHat based OS
			# so recycle the state of the slaves and master
			$self->recycle_bonded();
		} else {
			system("ifdown $dev >/dev/null 2>&1 ");
			sleep $wait; # VLAN environment require this sleep. bug3744.
			if ($ENV{'LK_DISTRIBUTION'} ne "suse") {
				system("ifconfig $dev up > /dev/null 2>&1");
			}
			system("ifup $dev >/dev/null 2>&1 ");
		}
		# check again
		my $cnt;
		for ($cnt=0; $cnt < $self->{'max_linkchk'}; $cnt++) {
			LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_DEBUG, 'ip', $0, $self->{'tag'}, 123023, "check link up: $cnt / $self->{'max_linkchk'}");
			sleep 1;
			last if (! $self->is_link_down);
		}
		return 1 if ($cnt == $self->{'max_linkchk'});
	}

	# See if the IP address is already configured
	if ($self->is_ip_configured) {
		if (! ($self->check())[0]) { # check returns 0 if OK
			# IP already exists
			return 0;
		} else {
			# IP already exists, but there is a problem
			return 1;
		}
	}

	# Virtual IP is somewhere?
	if((! $self->{'no_ip_unique'}) && ($self->ip_exists)) {
		# VIP is somewhere
		$self->vLogMsg(LK::LK_ERROR, 'ip', $0, 123024, 'IP address seems to still exist somewhere else.');
		return 1;
	}

	# create address
	LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_DEBUG, 'ip', $0, $self->{'tag'}, 123025, "ip -$self->{'ipversion'} addr add $self->{'ip'}/$self->{'prefix'} dev $self->{'device'}");
	my $result = system("ip -$self->{'ipversion'} addr add $self->{'ip'}/$self->{'prefix'} dev $self->{'device'}");
	if ($result == 0 || $result == 2) { # 0 success, 2 already exist.
		# flush arp cache
		foreach (IPutil::get_dev_list) {
			LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_DEBUG, 'ip', $0, $self->{'tag'}, 123026, "ip neigh flush dev $_");
			system("ip neigh flush dev $_ >/dev/null 2>&1");
		}
		# call IPv4/6 flush arp routine
		$self->{'ipint'}->flush_arp;
		$self->srcaddr(1) if $self->{'src_addr'};
		my ($checkerr, $usedbcast) = $self->check();
		if (! $checkerr) {
			return 0;
		} elsif ($usedbcast) {
			# the check failed and it tried a bcast ping, so yell...
			my $msg = "The broadcast ping used to determine the viability of $self->{'ip'}/$self->{'prefix'} failed.  "
				. "Please ensure that the ping list for this resource is properly configured in the properties "
				. "panel or that broadcast ping checking is disabled by adding NOBCASTPING=1 to the "
				. "/etc/default/LifeKeeper configuration file.";
			$self->vLogMsg(LK::LK_ERROR, 'ip', $0, 123096, $msg);
		}
	} else {
		return 1;
	}

	# remove virtual IP address if restore failed
	$self->App_remove;
	return 1;
}

sub App_recover {
	my $self = shift;

	# check real ip resource
	if ($self->{'realip'}) {
		LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_DEBUG, 'ip', $0, $self->{'tag'}, 123099, "real ip resource");
		return 1; # error
	}

	# call remove and restore
	$self->App_remove;
	return $self->App_restore;
}

sub App_quickCheck {
	my $self = shift;
	my ($ClassRef, $EventRef, $quickCheckYourAppRef) = @_;
	# Set up return values.
	$$ClassRef = 'ip';
	$$EventRef = 'pingfail';
	$$quickCheckYourAppRef = 'quickCheckIP';

	# check restore mode
	if ($self->{'restore_mode'}) {
		return 0;
	}

	# signal handler ($SIG{ALRM}) is set by RKActionHandler.pm
	alarm $self->{'quick_check_timeout'};
	return ($self->check())[0]; # return code is first element of array
}

sub App_remove {
	my $self = shift;
	my $ver = $self->{'ipversion'};
	my $ret;
	LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_DEBUG, 'ip', $0, $self->{'tag'}, 123028, "In IPint::App_remove");

	$self->{'prefix'} = 32 if(on_aws());

	# check real ip resource
	if ($self->{'realip'}) {
		LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_DEBUG, 'ip', $0, $self->{'tag'}, 123100, "real ip resource");
		return 0;
	}

	if (! $self->is_ip_configured) {
		# already removed
		return 0;
	}
	$self->srcaddr(0) if $self->{'src_addr'};
	LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_DEBUG, 'ip', $0, $self->{'tag'}, 123029, "ip -$ver addr delete $self->{'ip'}/$self->{'prefix'} dev $self->{'device'}");
	$ret = system("ip -$ver addr delete $self->{'ip'}/$self->{'prefix'} dev $self->{'device'}");

	# Delete leftover host route in RHEL 8 (PL-4575)
	LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_DEBUG, 'ip', $0, $self->{'tag'}, 123027, "ip -$ver route delete $self->{'ip'}/$self->{'prefix'} dev $self->{'device'}");
	system("ip -$ver route delete $self->{'ip'}/$self->{'prefix'} dev $self->{'device'} > /dev/null 2>&1");

	system("echo 0 > /proc/sys/net/ipv$ver/route/flush 2>/dev/null ");
	return $ret;
}

sub App_delete {
	my $self = shift;

	LKBase::LogMsg({'LOGFILE' => 1}, LK::LK_DEBUG, 'ip', $0, $self->{'tag'}, 123030, "In IPint::App_delete, calling App_remove");
	system("rm -f $self->{'resmode_path'}");
	return $self->App_remove();
}

#
# These are accessors for properties
#

sub get_restoreMode {
	my $self = shift;

	if ( !$self->{'restore_mode'} ) {
		return "Enabled";
	} else {
		return "Disabled";
	}
}

sub set_restoreMode {
	my $self = shift;
	my $val = shift;
	my $ret = 1; # fail, until we know it worked

	$self->GetLocks();

	if( $val eq "Disabled" ) {
		system ("touch ".$self->{'resmode_path'} );
		$ret=0 if ( -f $self->{'resmode_path'} );
	} else {
		# Assume it's 'Enabled'
		unlink $self->{'resmode_path'};
		$ret=0 if ( ! -f $self->{'resmode_path'} );
	}

	$self->ReleaseLocks();

	return $ret;
}

1;
